package ink.wgink.service.core.service.manage.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.user.mongo.IMongoLoginUserService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.department.DepartmentDTO;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.pos.DepartmentPO;
import ink.wgink.pojo.pos.GroupPO;
import ink.wgink.pojo.pos.PositionPO;
import ink.wgink.pojo.pos.RolePO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.core.dao.manage.ICoreManageDao;
import ink.wgink.service.core.pojo.vos.manage.CoreManageUserVO;
import ink.wgink.service.core.service.manage.ICoreManageService;
import ink.wgink.service.department.service.IDepartmentService;
import ink.wgink.service.department.service.IDepartmentUserService;
import ink.wgink.service.group.service.IGroupUserService;
import ink.wgink.service.position.service.IPositionUserService;
import ink.wgink.service.role.service.IRoleUserService;
import ink.wgink.service.user.service.IUserService;
import ink.wgink.util.thread.CachedThreadPoolUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: CoreManageServiceImpl
 * @Description: 核心管理业务
 * @Author: wanggeng
 * @Date: 2022/8/10 16:28
 * @Version: 1.0
 */
@Service
public class CoreManageServiceImpl extends DefaultBaseService implements ICoreManageService {
    @Autowired
    private ICoreManageDao coreManageDao;
    @Autowired
    private IUserService userService;
    @Autowired
    private IDepartmentService departmentService;
    @Autowired
    private IDepartmentUserService departmentUserService;
    @Autowired
    private IRoleUserService roleUserService;
    @Autowired
    private IPositionUserService positionUserService;
    @Autowired
    private IGroupUserService groupUserService;
    @Autowired(required = false)
    private IMongoLoginUserService mongoLoginUserService;

    @Override
    public void saveUser(CoreManageUserVO coreManageUserVO) {
        String userId = userService.saveAndReturnId(coreManageUserVO, false);
        setUserServices(userId, coreManageUserVO);
        updateMongoLoginUser(userId);
    }

    @Override
    public void removeUser(List<String> userIds) {
        userService.remove(userIds);
    }

    @Override
    public void updateUser(String userId, CoreManageUserVO coreManageUserVO) {
        userService.update(userId, coreManageUserVO);
        setUserServices(userId, coreManageUserVO);
        updateMongoLoginUser(userId);
    }

    @Override
    public UserDTO getUser(String userId) {
        UserDTO userDTO = userService.get(userId);
        if (userDTO == null) {
            throw new SearchException("用户不存在");
        }
        setUserDepartments(userDTO);
        setUserRoles(userDTO);
        setUserPositions(userDTO);
        setUserGroups(userDTO);
        return userDTO;
    }

    @Override
    public List<DepartmentDTO> listDepartmentNavPathByDepartmentId(String departmentId) {
        DepartmentPO departmentPO = departmentService.getPO(departmentId);
        if (departmentPO == null) {
            throw new SearchException("部门不存在");
        }
        List<DepartmentDTO> navList = new ArrayList<>();
        setDepartmentNav(navList, departmentPO);
        return navList;
    }

    @Override
    public List<UserDTO> listUser(Map<String, Object> params) {
        params = params == null ? getHashMap(0) : params;
        return coreManageDao.listUser(params);
    }

    @Override
    public SuccessResultList<List<UserDTO>> listPageUser(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<UserDTO> userDTOs = listUser(page.getParams());
        PageInfo<UserDTO> pageInfo = new PageInfo<>(userDTOs);
        return new SuccessResultList<>(userDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    /**
     * 设置部门导航
     *
     * @param navList
     * @param departmentPO
     */
    private void setDepartmentNav(List<DepartmentDTO> navList, DepartmentPO departmentPO) {
        DepartmentDTO departmentDTO = new DepartmentDTO();
        BeanUtils.copyProperties(departmentPO, departmentDTO);
        navList.add(0, departmentDTO);
        if (StringUtils.equals(ISystemConstant.TREE_ROOT_ID, departmentPO.getDepartmentParentId())) {
            return;
        }
        DepartmentPO departmentParentPO = departmentService.getPO(departmentPO.getDepartmentParentId());
        setDepartmentNav(navList, departmentParentPO);
    }

    /**
     * 设置用户业务
     *
     * @param userId
     * @param coreManageUserVO
     */
    private void setUserServices(String userId, CoreManageUserVO coreManageUserVO) {
        List<String> departmentIds = coreManageUserVO.getDepartmentIds();
        departmentUserService.saveAfterDeleteByUserId(departmentIds, userId);
        List<String> roleIds = coreManageUserVO.getRoleIds();
        roleUserService.saveAfterDeleteByUserId(roleIds, userId);
        List<String> positionIds = coreManageUserVO.getPositionIds();
        positionUserService.saveAfterDeleteByUserId(positionIds, userId);
        List<String> groupIds = coreManageUserVO.getGroupIds();
        groupUserService.saveAfterDeleteByUserId(groupIds, userId);
    }

    /**
     * 设置用户部门
     *
     * @param userDTO
     */
    private void setUserDepartments(UserDTO userDTO) {
        List<DepartmentPO> departmentPOS = departmentUserService.listDepartmentPOByUserId(userDTO.getUserId());
        String departmentIds = "";
        String departmentNames = "";
        for (DepartmentPO departmentPO : departmentPOS) {
            if (!StringUtils.isBlank(departmentIds)) {
                departmentIds += ",";
                departmentNames += ",";
            }
            departmentIds += departmentPO.getDepartmentId();
            departmentNames += departmentPO.getDepartmentName();
        }
        userDTO.setDepartmentIds(departmentIds);
        userDTO.setDepartmentNames(departmentNames);
    }

    /**
     * 设置用户角色
     *
     * @param userDTO
     */
    private void setUserRoles(UserDTO userDTO) {
        List<RolePO> rolePOS = roleUserService.listRolePOByUserId(userDTO.getUserId());
        String roleIds = "";
        String roleNames = "";
        for (RolePO rolePO : rolePOS) {
            if (!StringUtils.isBlank(roleIds)) {
                roleIds += ",";
                roleNames += ",";
            }
            roleIds += rolePO.getRoleId();
            roleNames += rolePO.getRoleName();
        }
        userDTO.setRoleIds(roleIds);
        userDTO.setRoleNames(roleNames);
    }

    /**
     * 设置用户职位
     *
     * @param userDTO
     */
    private void setUserPositions(UserDTO userDTO) {
        List<PositionPO> positionPOS = positionUserService.listPositionPOByUserId(userDTO.getUserId());
        String positionIds = "";
        String positionNames = "";
        for (PositionPO positionPO : positionPOS) {
            if (!StringUtils.isBlank(positionIds)) {
                positionIds += ",";
                positionNames += ",";
            }
            positionIds += positionPO.getPositionId();
            positionNames += positionPO.getPositionName();
        }
        userDTO.setPositionIds(positionIds);
        userDTO.setPositionNames(positionNames);
    }

    /**
     * 设置用户组
     *
     * @param userDTO
     */
    private void setUserGroups(UserDTO userDTO) {
        List<GroupPO> groupPOS = groupUserService.listGroupPOByUserId(userDTO.getUserId());
        String groupIds = "";
        String groupNames = "";
        for (GroupPO groupPO : groupPOS) {
            if (!StringUtils.isBlank(groupIds)) {
                groupIds += ",";
                groupNames += ",";
            }
            groupIds += groupPO.getGroupId();
            groupNames += groupPO.getGroupName();
        }
        userDTO.setGroupIds(groupIds);
        userDTO.setGroupNames(groupNames);
    }

    /**
     * 更新mongo登录用户
     *
     * @param userId
     */
    private void updateMongoLoginUser(String userId) {
        if (mongoLoginUserService != null) {
            CachedThreadPoolUtil.execute(() -> {
                mongoLoginUserService.updateByUserId(userId);
            });
        }
    }
}
